package javax.swing.plaf.basic;

import javax.swing.BoundedRangeModel;
import javax.swing.Scrollable;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.awt.event.MouseWheelEvent;
import javax.swing.event.ChangeEvent;
import java.beans.PropertyChangeEvent;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Point;
import sun.swing.DefaultLookup;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import javax.swing.JScrollBar;
import javax.swing.JViewport;
import javax.swing.UIManager;
import javax.swing.plaf.UIResource;
import javax.swing.LookAndFeel;
import java.awt.Dimension;
import java.awt.Rectangle;
import javax.swing.border.Border;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.event.MouseWheelListener;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeListener;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.plaf.ScrollPaneUI;

public class BasicScrollPaneUI extends ScrollPaneUI implements ScrollPaneConstants
{
    protected JScrollPane scrollpane;
    protected ChangeListener vsbChangeListener;
    protected ChangeListener hsbChangeListener;
    protected ChangeListener viewportChangeListener;
    protected PropertyChangeListener spPropertyChangeListener;
    private MouseWheelListener mouseScrollListener;
    private int oldExtent;
    private PropertyChangeListener vsbPropertyChangeListener;
    private PropertyChangeListener hsbPropertyChangeListener;
    private BasicScrollPaneUI.Handler handler;
    private boolean setValueCalled;
    
    public BasicScrollPaneUI() {
        this.oldExtent = Integer.MIN_VALUE;
        this.setValueCalled = false;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicScrollPaneUI();
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicScrollPaneUI.Actions("scrollUp"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("scrollDown"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("scrollHome"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("scrollEnd"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("unitScrollUp"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("unitScrollDown"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("scrollLeft"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("scrollRight"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("unitScrollRight"));
        lazyActionMap.put(new BasicScrollPaneUI.Actions("unitScrollLeft"));
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        Border viewportBorder = this.scrollpane.getViewportBorder();
        if (viewportBorder != null) {
            Rectangle viewportBorderBounds = this.scrollpane.getViewportBorderBounds();
            viewportBorder.paintBorder(this.scrollpane, graphics, viewportBorderBounds.x, viewportBorderBounds.y, viewportBorderBounds.width, viewportBorderBounds.height);
        }
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        return new Dimension(32767, 32767);
    }
    
    protected void installDefaults(JScrollPane c) {
        LookAndFeel.installBorder(c, "ScrollPane.border");
        LookAndFeel.installColorsAndFont(c, "ScrollPane.background", "ScrollPane.foreground", "ScrollPane.font");
        Border viewportBorder = c.getViewportBorder();
        if (viewportBorder == null || viewportBorder instanceof UIResource) {
            c.setViewportBorder(UIManager.getBorder("ScrollPane.viewportBorder"));
        }
        LookAndFeel.installProperty(c, "opaque", Boolean.TRUE);
    }
    
    protected void installListeners(JScrollPane scrollPane) {
        this.vsbChangeListener = this.createVSBChangeListener();
        this.vsbPropertyChangeListener = this.createVSBPropertyChangeListener();
        this.hsbChangeListener = this.createHSBChangeListener();
        this.hsbPropertyChangeListener = this.createHSBPropertyChangeListener();
        this.viewportChangeListener = this.createViewportChangeListener();
        this.spPropertyChangeListener = this.createPropertyChangeListener();
        JViewport viewport = this.scrollpane.getViewport();
        JScrollBar verticalScrollBar = this.scrollpane.getVerticalScrollBar();
        JScrollBar horizontalScrollBar = this.scrollpane.getHorizontalScrollBar();
        if (viewport != null) {
            viewport.addChangeListener(this.viewportChangeListener);
        }
        if (verticalScrollBar != null) {
            verticalScrollBar.getModel().addChangeListener(this.vsbChangeListener);
            verticalScrollBar.addPropertyChangeListener(this.vsbPropertyChangeListener);
        }
        if (horizontalScrollBar != null) {
            horizontalScrollBar.getModel().addChangeListener(this.hsbChangeListener);
            horizontalScrollBar.addPropertyChangeListener(this.hsbPropertyChangeListener);
        }
        this.scrollpane.addPropertyChangeListener(this.spPropertyChangeListener);
        this.mouseScrollListener = this.createMouseWheelListener();
        this.scrollpane.addMouseWheelListener(this.mouseScrollListener);
    }
    
    protected void installKeyboardActions(JScrollPane component) {
        SwingUtilities.replaceUIInputMap(component, 1, this.getInputMap(1));
        LazyActionMap.installLazyActionMap(component, BasicScrollPaneUI.class, "ScrollPane.actionMap");
    }
    
    InputMap getInputMap(int n) {
        if (n == 1) {
            InputMap parent = (InputMap)DefaultLookup.get(this.scrollpane, this, "ScrollPane.ancestorInputMap");
            InputMap inputMap;
            if (this.scrollpane.getComponentOrientation().isLeftToRight() || (inputMap = (InputMap)DefaultLookup.get(this.scrollpane, this, "ScrollPane.ancestorInputMap.RightToLeft")) == null) {
                return parent;
            }
            inputMap.setParent(parent);
            return inputMap;
        }
        else {
            return null;
        }
    }
    
    @Override
    public void installUI(JComponent component) {
        this.installDefaults(this.scrollpane = (JScrollPane)component);
        this.installListeners(this.scrollpane);
        this.installKeyboardActions(this.scrollpane);
    }
    
    protected void uninstallDefaults(JScrollPane scrollPane) {
        LookAndFeel.uninstallBorder(this.scrollpane);
        if ((this.scrollpane.getViewportBorder() instanceof UIResource)) {
            this.scrollpane.setViewportBorder(null);
        }
    }
    
    protected void uninstallListeners(JComponent component) {
        JViewport viewport = this.scrollpane.getViewport();
        JScrollBar verticalScrollBar = this.scrollpane.getVerticalScrollBar();
        JScrollBar horizontalScrollBar = this.scrollpane.getHorizontalScrollBar();
        if (viewport != null) {
            viewport.removeChangeListener(this.viewportChangeListener);
        }
        if (verticalScrollBar != null) {
            verticalScrollBar.getModel().removeChangeListener(this.vsbChangeListener);
            verticalScrollBar.removePropertyChangeListener(this.vsbPropertyChangeListener);
        }
        if (horizontalScrollBar != null) {
            horizontalScrollBar.getModel().removeChangeListener(this.hsbChangeListener);
            horizontalScrollBar.removePropertyChangeListener(this.hsbPropertyChangeListener);
        }
        this.scrollpane.removePropertyChangeListener(this.spPropertyChangeListener);
        if (this.mouseScrollListener != null) {
            this.scrollpane.removeMouseWheelListener(this.mouseScrollListener);
        }
        this.vsbChangeListener = null;
        this.hsbChangeListener = null;
        this.viewportChangeListener = null;
        this.spPropertyChangeListener = null;
        this.mouseScrollListener = null;
        this.handler = null;
    }
    
    protected void uninstallKeyboardActions(JScrollPane scrollPane) {
        SwingUtilities.replaceUIActionMap(scrollPane, null);
        SwingUtilities.replaceUIInputMap(scrollPane, 1, null);
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults(this.scrollpane);
        this.uninstallListeners(this.scrollpane);
        this.uninstallKeyboardActions(this.scrollpane);
        this.scrollpane = null;
    }
    
    private BasicScrollPaneUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicScrollPaneUI.Handler();
        }
        return this.handler;
    }
    
    protected void syncScrollPaneWithViewport() {
        JViewport viewport = this.scrollpane.getViewport();
        JScrollBar verticalScrollBar = this.scrollpane.getVerticalScrollBar();
        JScrollBar horizontalScrollBar = this.scrollpane.getHorizontalScrollBar();
        JViewport rowHeader = this.scrollpane.getRowHeader();
        JViewport columnHeader = this.scrollpane.getColumnHeader();
        int leftToRight = this.scrollpane.getComponentOrientation().isLeftToRight();
        if (viewport != null) {
            Dimension extentSize = viewport.getExtentSize();
            Dimension viewSize = viewport.getViewSize();
            Point viewPosition = viewport.getViewPosition();
            if (verticalScrollBar != null) {
                int height = extentSize.height;
                int height2 = viewSize.height;
                verticalScrollBar.setValues(Math.max(0, Math.min(viewPosition.y, height2 - height)), height, 0, height2);
            }
            if (horizontalScrollBar != null) {
                int width = extentSize.width;
                int width2 = viewSize.width;
                int newValue;
                if (!(leftToRight == 0)) {
                    newValue = Math.max(0, Math.min(viewPosition.x, width2 - width));
                }
                else {
                    int value = horizontalScrollBar.getValue();
                    if (this.setValueCalled && width2 - value == viewPosition.x) {
                        newValue = Math.max(0, Math.min(width2 - width, value));
                        if (width != 0) {
                            this.setValueCalled = false;
                        }
                    }
                    else if (width > width2) {
                        viewPosition.x = width2 - width;
                        viewport.setViewPosition(viewPosition);
                        newValue = 0;
                    }
                    else {
                        newValue = Math.max(0, Math.min(width2 - width, width2 - width - viewPosition.x));
                        if (this.oldExtent > width) {
                            newValue -= this.oldExtent - width;
                        }
                    }
                }
                horizontalScrollBar.setValues(newValue, this.oldExtent = width, 0, width2);
            }
            if (rowHeader != null) {
                Point viewPosition2 = rowHeader.getViewPosition();
                viewPosition2.y = viewport.getViewPosition().y;
                viewPosition2.x = 0;
                rowHeader.setViewPosition(viewPosition2);
            }
            if (columnHeader != null) {
                Point viewPosition3 = columnHeader.getViewPosition();
                if (!(leftToRight == 0)) {
                    viewPosition3.x = viewport.getViewPosition().x;
                }
                else {
                    viewPosition3.x = Math.max(0, viewport.getViewPosition().x);
                }
                viewPosition3.y = 0;
                columnHeader.setViewPosition(viewPosition3);
            }
        }
    }
    
    @Override
    public int getBaseline(JComponent component, int max, int max2) {
        if (component == null) {
            throw new NullPointerException("Component must be non-null");
        }
        if (max < 0 || max2 < 0) {
            throw new IllegalArgumentException("Width and height must be >= 0");
        }
        JViewport viewport = this.scrollpane.getViewport();
        Insets insets = this.scrollpane.getInsets();
        int top = insets.top;
        max2 = max2 - insets.top - insets.bottom;
        max = max - insets.left - insets.right;
        JViewport columnHeader = this.scrollpane.getColumnHeader();
        if (columnHeader != null && columnHeader.isVisible()) {
            Component view = columnHeader.getView();
            if (view != null && view.isVisible()) {
                Dimension preferredSize = view.getPreferredSize();
                int baseline = view.getBaseline(preferredSize.width, preferredSize.height);
                if (baseline >= 0) {
                    return top + baseline;
                }
            }
            Dimension preferredSize2 = columnHeader.getPreferredSize();
            max2 -= preferredSize2.height;
            top += preferredSize2.height;
        }
        Component component2 = (viewport == null) ? null : viewport.getView();
        if (component2 != null && component2.isVisible() && component2.getBaselineResizeBehavior() == Component.BaselineResizeBehavior.CONSTANT_ASCENT) {
            Border viewportBorder = this.scrollpane.getViewportBorder();
            if (viewportBorder != null) {
                Insets borderInsets = viewportBorder.getBorderInsets(this.scrollpane);
                top += borderInsets.top;
                max2 = max2 - borderInsets.top - borderInsets.bottom;
                max = max - borderInsets.left - borderInsets.right;
            }
            if (component2.getWidth() > 0 && component2.getHeight() > 0) {
                Dimension minimumSize = component2.getMinimumSize();
                max = Math.max(minimumSize.width, component2.getWidth());
                max2 = Math.max(minimumSize.height, component2.getHeight());
            }
            if (max > 0 && max2 > 0) {
                int baseline2 = component2.getBaseline(max, max2);
                if (baseline2 > 0) {
                    return top + baseline2;
                }
            }
        }
        return -1;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        return Component.BaselineResizeBehavior.CONSTANT_ASCENT;
    }
    
    protected ChangeListener createViewportChangeListener() {
        return this.getHandler();
    }
    
    private PropertyChangeListener createHSBPropertyChangeListener() {
        return this.getHandler();
    }
    
    protected ChangeListener createHSBChangeListener() {
        return this.getHandler();
    }
    
    private PropertyChangeListener createVSBPropertyChangeListener() {
        return this.getHandler();
    }
    
    protected ChangeListener createVSBChangeListener() {
        return this.getHandler();
    }
    
    protected MouseWheelListener createMouseWheelListener() {
        return this.getHandler();
    }
    
    protected void updateScrollBarDisplayPolicy(PropertyChangeEvent propertyChangeEvent) {
        this.scrollpane.revalidate();
        this.scrollpane.repaint();
    }
    
    protected void updateViewport(PropertyChangeEvent propertyChangeEvent) {
        JViewport viewport = (JViewport)(JViewport)propertyChangeEvent.getOldValue();
        JViewport viewport2 = (JViewport)(JViewport)propertyChangeEvent.getNewValue();
        if (viewport != null) {
            viewport.removeChangeListener(this.viewportChangeListener);
        }
        if (viewport2 != null) {
            Point viewPosition = viewport2.getViewPosition();
            if (this.scrollpane.getComponentOrientation().isLeftToRight()) {
                viewPosition.x = Math.max(viewPosition.x, 0);
            }
            else {
                int width = viewport2.getViewSize().width;
                int width2 = viewport2.getExtentSize().width;
                if (width2 > width) {
                    viewPosition.x = width - width2;
                }
                else {
                    viewPosition.x = Math.max(0, Math.min(width - width2, viewPosition.x));
                }
            }
            viewPosition.y = Math.max(viewPosition.y, 0);
            viewport2.setViewPosition(viewPosition);
            viewport2.addChangeListener(this.viewportChangeListener);
        }
    }
    
    protected void updateRowHeader(PropertyChangeEvent propertyChangeEvent) {
        JViewport viewport = (JViewport)(JViewport)propertyChangeEvent.getNewValue();
        if (viewport != null) {
            JViewport viewport2 = this.scrollpane.getViewport();
            Point viewPosition = viewport.getViewPosition();
            viewPosition.y = ((viewport2 != null) ? viewport2.getViewPosition().y : 0);
            viewport.setViewPosition(viewPosition);
        }
    }
    
    protected void updateColumnHeader(PropertyChangeEvent propertyChangeEvent) {
        JViewport comp = (JViewport)(JViewport)propertyChangeEvent.getNewValue();
        if (comp != null) {
            JViewport viewport = this.scrollpane.getViewport();
            Point viewPosition = comp.getViewPosition();
            if (viewport == null) {
                viewPosition.x = 0;
            }
            else if (this.scrollpane.getComponentOrientation().isLeftToRight()) {
                viewPosition.x = viewport.getViewPosition().x;
            }
            else {
                viewPosition.x = Math.max(0, viewport.getViewPosition().x);
            }
            comp.setViewPosition(viewPosition);
            this.scrollpane.add(comp, "COLUMN_HEADER");
        }
    }
    
    private void updateHorizontalScrollBar(PropertyChangeEvent propertyChangeEvent) {
        this.updateScrollBar(propertyChangeEvent, this.hsbChangeListener, this.hsbPropertyChangeListener);
    }
    
    private void updateVerticalScrollBar(PropertyChangeEvent propertyChangeEvent) {
        this.updateScrollBar(propertyChangeEvent, this.vsbChangeListener, this.vsbPropertyChangeListener);
    }
    
    private void updateScrollBar(PropertyChangeEvent propertyChangeEvent, ChangeListener changeListener, PropertyChangeListener propertyChangeListener) {
        JScrollBar scrollBar = (JScrollBar)propertyChangeEvent.getOldValue();
        if (scrollBar != null) {
            if (changeListener != null) {
                scrollBar.getModel().removeChangeListener(changeListener);
            }
            if (propertyChangeListener != null) {
                scrollBar.removePropertyChangeListener(propertyChangeListener);
            }
        }
        JScrollBar scrollBar2 = (JScrollBar)propertyChangeEvent.getNewValue();
        if (scrollBar2 != null) {
            if (changeListener != null) {
                scrollBar2.getModel().addChangeListener(changeListener);
            }
            if (propertyChangeListener != null) {
                scrollBar2.addPropertyChangeListener(propertyChangeListener);
            }
        }
    }
    
    protected PropertyChangeListener createPropertyChangeListener() {
        return this.getHandler();
    }
    
    static BasicScrollPaneUI.Handler access$000(BasicScrollPaneUI basicScrollPaneUI) {
        return basicScrollPaneUI.getHandler();
    }
    
    static void access$200(BasicScrollPaneUI basicScrollPaneUI, PropertyChangeEvent propertyChangeEvent) {
        basicScrollPaneUI.updateVerticalScrollBar(propertyChangeEvent);
    }
    
    static void access$300(BasicScrollPaneUI basicScrollPaneUI, PropertyChangeEvent propertyChangeEvent) {
        basicScrollPaneUI.updateHorizontalScrollBar(propertyChangeEvent);
    }
    
    public class ViewportChangeHandler implements ChangeListener
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            BasicScrollPaneUI.access$000(BasicScrollPaneUI.this).stateChanged(changeEvent);
        }
    }
    
    public class HSBChangeListener implements ChangeListener
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            BasicScrollPaneUI.access$000(BasicScrollPaneUI.this).stateChanged(changeEvent);
        }
    }
    
    public class VSBChangeListener implements ChangeListener
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            BasicScrollPaneUI.access$000(BasicScrollPaneUI.this).stateChanged(changeEvent);
        }
    }
    
    protected class MouseWheelHandler implements MouseWheelListener
    {
        @Override
        public void mouseWheelMoved(MouseWheelEvent mouseWheelEvent) {
            BasicScrollPaneUI.access$000(BasicScrollPaneUI.this).mouseWheelMoved(mouseWheelEvent);
        }
    }
    
    public class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            BasicScrollPaneUI.access$000(BasicScrollPaneUI.this).propertyChange(propertyChangeEvent);
        }
    }
    
    private static class Actions extends UIAction
    {
        private static final String SCROLL_UP = "scrollUp";
        private static final String SCROLL_DOWN = "scrollDown";
        private static final String SCROLL_HOME = "scrollHome";
        private static final String SCROLL_END = "scrollEnd";
        private static final String UNIT_SCROLL_UP = "unitScrollUp";
        private static final String UNIT_SCROLL_DOWN = "unitScrollDown";
        private static final String SCROLL_LEFT = "scrollLeft";
        private static final String SCROLL_RIGHT = "scrollRight";
        private static final String UNIT_SCROLL_LEFT = "unitScrollLeft";
        private static final String UNIT_SCROLL_RIGHT = "unitScrollRight";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JScrollPane scrollPane = (JScrollPane)actionEvent.getSource();
            int leftToRight = scrollPane.getComponentOrientation().isLeftToRight();
            String name = this.getName();
            if (name == "scrollUp") {
                this.scroll(scrollPane, 1, -1, true);
            }
            else if (name == "scrollDown") {
                this.scroll(scrollPane, 1, 1, true);
            }
            else if (name == "scrollHome") {
                this.scrollHome(scrollPane);
            }
            else if (name == "scrollEnd") {
                this.scrollEnd(scrollPane);
            }
            else if (name == "unitScrollUp") {
                this.scroll(scrollPane, 1, -1, false);
            }
            else if (name == "unitScrollDown") {
                this.scroll(scrollPane, 1, 1, false);
            }
            else if (name == "scrollLeft") {
                this.scroll(scrollPane, 0, leftToRight ? -1 : 1, true);
            }
            else if (name == "scrollRight") {
                this.scroll(scrollPane, 0, leftToRight ? 1 : -1, true);
            }
            else if (name == "unitScrollLeft") {
                this.scroll(scrollPane, 0, leftToRight ? -1 : 1, false);
            }
            else if (name == "unitScrollRight") {
                this.scroll(scrollPane, 0, leftToRight ? 1 : -1, false);
            }
        }
        
        private void scrollEnd(JScrollPane scrollPane) {
            JViewport viewport = scrollPane.getViewport();
            Component view;
            if (viewport != null && (view = viewport.getView()) != null) {
                Rectangle viewRect = viewport.getViewRect();
                Rectangle bounds = view.getBounds();
                if (scrollPane.getComponentOrientation().isLeftToRight()) {
                    viewport.setViewPosition(new Point(bounds.width - viewRect.width, bounds.height - viewRect.height));
                }
                else {
                    viewport.setViewPosition(new Point(0, bounds.height - viewRect.height));
                }
            }
        }
        
        private void scrollHome(JScrollPane scrollPane) {
            JViewport viewport = scrollPane.getViewport();
            Component view;
            if (viewport != null && (view = viewport.getView()) != null) {
                if (scrollPane.getComponentOrientation().isLeftToRight()) {
                    viewport.setViewPosition(new Point(0, 0));
                }
                else {
                    viewport.setViewPosition(new Point(view.getBounds().width - viewport.getViewRect().width, 0));
                }
            }
        }
        
        private void scroll(JScrollPane scrollPane, int n, int n2, boolean b) {
            JViewport viewport = scrollPane.getViewport();
            Component view;
            if (viewport != null && (view = viewport.getView()) != null) {
                Rectangle viewRect = viewport.getViewRect();
                Dimension size = view.getSize();
                int n3;
                if ((view instanceof Scrollable)) {
                    if (b) {
                        n3 = ((Scrollable)view).getScrollableBlockIncrement(viewRect, n, n2);
                    }
                    else {
                        n3 = ((Scrollable)view).getScrollableUnitIncrement(viewRect, n, n2);
                    }
                }
                else if (b) {
                    if (n == 1) {
                        n3 = viewRect.height;
                    }
                    else {
                        n3 = viewRect.width;
                    }
                }
                else {
                    n3 = 10;
                }
                if (n == 1) {
                    viewRect.y += n3 * n2;
                    if (viewRect.y + viewRect.height > size.height) {
                        viewRect.y = Math.max(0, size.height - viewRect.height);
                    }
                    else if (viewRect.y < 0) {
                        viewRect.y = 0;
                    }
                }
                else if (scrollPane.getComponentOrientation().isLeftToRight()) {
                    viewRect.x += n3 * n2;
                    if (viewRect.x + viewRect.width > size.width) {
                        viewRect.x = Math.max(0, size.width - viewRect.width);
                    }
                    else if (viewRect.x < 0) {
                        viewRect.x = 0;
                    }
                }
                else {
                    viewRect.x -= n3 * n2;
                    if (viewRect.width > size.width) {
                        viewRect.x = size.width - viewRect.width;
                    }
                    else {
                        viewRect.x = Math.max(0, Math.min(size.width - viewRect.width, viewRect.x));
                    }
                }
                viewport.setViewPosition(viewRect.getLocation());
            }
        }
    }
    
    class Handler implements ChangeListener, PropertyChangeListener, MouseWheelListener
    {
        @Override
        public void mouseWheelMoved(MouseWheelEvent mouseWheelEvent) {
            if (BasicScrollPaneUI.this.scrollpane.isWheelScrollingEnabled() && mouseWheelEvent.getWheelRotation() != 0) {
                JScrollBar scrollBar = BasicScrollPaneUI.this.scrollpane.getVerticalScrollBar();
                int n = (mouseWheelEvent.getWheelRotation() < 0) ? -1 : 1;
                int n2 = 1;
                if (scrollBar == null || !scrollBar.isVisible() || mouseWheelEvent.isShiftDown()) {
                    scrollBar = BasicScrollPaneUI.this.scrollpane.getHorizontalScrollBar();
                    if (scrollBar == null || !scrollBar.isVisible()) {
                        return;
                    }
                    n2 = 0;
                }
                mouseWheelEvent.consume();
                if (mouseWheelEvent.getScrollType() == 0) {
                    JViewport viewport = BasicScrollPaneUI.this.scrollpane.getViewport();
                    if (viewport == null) {
                        return;
                    }
                    Component view = viewport.getView();
                    int abs = Math.abs(mouseWheelEvent.getUnitsToScroll());
                    int n3 = Math.abs(mouseWheelEvent.getWheelRotation()) == 1;
                    if (Boolean.TRUE == scrollBar.getClientProperty("JScrollBar.fastWheelScrolling") && view instanceof Scrollable) {
                        Scrollable scrollable = (Scrollable)view;
                        Rectangle viewRect = viewport.getViewRect();
                        int x = viewRect.x;
                        int leftToRight = view.getComponentOrientation().isLeftToRight();
                        int y = scrollBar.getMinimum();
                        int min = scrollBar.getMaximum() - scrollBar.getModel().getExtent();
                        if (!(n3 == 0)) {
                            int scrollableBlockIncrement = scrollable.getScrollableBlockIncrement(viewRect, n2, n);
                            if (n < 0) {
                                y = Math.max(y, scrollBar.getValue() - scrollableBlockIncrement);
                            }
                            else {
                                min = Math.min(min, scrollBar.getValue() + scrollableBlockIncrement);
                            }
                        }
                        for (int i = 0; i < abs; ++i) {
                            int scrollableUnitIncrement = scrollable.getScrollableUnitIncrement(viewRect, n2, n);
                            if (n2 == 1) {
                                if (n < 0) {
                                    viewRect.y -= scrollableUnitIncrement;
                                    if (viewRect.y <= y) {
                                        viewRect.y = y;
                                        break;
                                    }
                                }
                                else {
                                    viewRect.y += scrollableUnitIncrement;
                                    if (viewRect.y >= min) {
                                        viewRect.y = min;
                                        break;
                                    }
                                }
                            }
                            else if ((leftToRight && n < 0) || (leftToRight == 0 && n > 0)) {
                                viewRect.x -= scrollableUnitIncrement;
                                if (leftToRight && viewRect.x < y) {
                                    viewRect.x = y;
                                    break;
                                }
                            }
                            else if ((leftToRight && n > 0) || (leftToRight == 0 && n < 0)) {
                                viewRect.x += scrollableUnitIncrement;
                                if (leftToRight && viewRect.x > min) {
                                    viewRect.x = min;
                                    break;
                                }
                            }
                            else
                                assert false : "Non-sensical ComponentOrientation / scroll direction";
                        }
                        if (n2 == 1) {
                            scrollBar.setValue(viewRect.y);
                        }
                        else if (!(leftToRight == 0)) {
                            scrollBar.setValue(viewRect.x);
                        }
                        else {
                            int value = scrollBar.getValue() - (viewRect.x - x);
                            if (value < y) {
                                value = y;
                            }
                            else if (value > min) {
                                value = min;
                            }
                            scrollBar.setValue(value);
                        }
                    }
                    else {
                        BasicScrollBarUI.scrollByUnits(scrollBar, n, abs, (boolean)n3);
                    }
                }
                else if (mouseWheelEvent.getScrollType() == 1) {
                    BasicScrollBarUI.scrollByBlock(scrollBar, n);
                }
            }
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            JViewport viewport = BasicScrollPaneUI.this.scrollpane.getViewport();
            if (viewport != null) {
                if (changeEvent.getSource() == viewport) {
                    BasicScrollPaneUI.this.syncScrollPaneWithViewport();
                }
                else {
                    JScrollBar horizontalScrollBar = BasicScrollPaneUI.this.scrollpane.getHorizontalScrollBar();
                    if (horizontalScrollBar != null && changeEvent.getSource() == horizontalScrollBar.getModel()) {
                        this.hsbStateChanged(viewport, changeEvent);
                    }
                    else {
                        JScrollBar verticalScrollBar = BasicScrollPaneUI.this.scrollpane.getVerticalScrollBar();
                        if (verticalScrollBar != null && changeEvent.getSource() == verticalScrollBar.getModel()) {
                            this.vsbStateChanged(viewport, changeEvent);
                        }
                    }
                }
            }
        }
        
        private void vsbStateChanged(JViewport viewport, ChangeEvent changeEvent) {
            BoundedRangeModel boundedRangeModel = (BoundedRangeModel)(BoundedRangeModel)changeEvent.getSource();
            Point viewPosition = viewport.getViewPosition();
            viewPosition.y = boundedRangeModel.getValue();
            viewport.setViewPosition(viewPosition);
        }
        
        private void hsbStateChanged(JViewport viewport, ChangeEvent changeEvent) {
            BoundedRangeModel boundedRangeModel = (BoundedRangeModel)(BoundedRangeModel)changeEvent.getSource();
            Point viewPosition = viewport.getViewPosition();
            int value = boundedRangeModel.getValue();
            if (BasicScrollPaneUI.this.scrollpane.getComponentOrientation().isLeftToRight()) {
                viewPosition.x = value;
            }
            else {
                int width = viewport.getViewSize().width;
                int width2 = viewport.getExtentSize().width;
                int x = viewPosition.x;
                viewPosition.x = width - width2 - value;
                if (width2 == 0 && value != 0 && x == width) {
                    BasicScrollPaneUI.this.setValueCalled = true;
                }
                else if (width2 != 0 && x < 0 && viewPosition.x == 0) {
                    viewPosition.x += value;
                }
            }
            viewport.setViewPosition(viewPosition);
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (propertyChangeEvent.getSource() == BasicScrollPaneUI.this.scrollpane) {
                this.scrollPanePropertyChange(propertyChangeEvent);
            }
            else {
                this.sbPropertyChange(propertyChangeEvent);
            }
        }
        
        private void scrollPanePropertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if (propertyName == "verticalScrollBarDisplayPolicy") {
                BasicScrollPaneUI.this.updateScrollBarDisplayPolicy(propertyChangeEvent);
            }
            else if (propertyName == "horizontalScrollBarDisplayPolicy") {
                BasicScrollPaneUI.this.updateScrollBarDisplayPolicy(propertyChangeEvent);
            }
            else if (propertyName == "viewport") {
                BasicScrollPaneUI.this.updateViewport(propertyChangeEvent);
            }
            else if (propertyName == "rowHeader") {
                BasicScrollPaneUI.this.updateRowHeader(propertyChangeEvent);
            }
            else if (propertyName == "columnHeader") {
                BasicScrollPaneUI.this.updateColumnHeader(propertyChangeEvent);
            }
            else if (propertyName == "verticalScrollBar") {
                BasicScrollPaneUI.access$200(BasicScrollPaneUI.this, propertyChangeEvent);
            }
            else if (propertyName == "horizontalScrollBar") {
                BasicScrollPaneUI.access$300(BasicScrollPaneUI.this, propertyChangeEvent);
            }
            else if (propertyName == "componentOrientation") {
                BasicScrollPaneUI.this.scrollpane.revalidate();
                BasicScrollPaneUI.this.scrollpane.repaint();
            }
        }
        
        private void sbPropertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            Object source = propertyChangeEvent.getSource();
            if ("model" == propertyName) {
                JScrollBar scrollBar = BasicScrollPaneUI.this.scrollpane.getVerticalScrollBar();
                BoundedRangeModel boundedRangeModel = (BoundedRangeModel)propertyChangeEvent.getOldValue();
                ChangeListener changeListener = null;
                if (source == scrollBar) {
                    changeListener = BasicScrollPaneUI.this.vsbChangeListener;
                }
                else if (source == BasicScrollPaneUI.this.scrollpane.getHorizontalScrollBar()) {
                    scrollBar = BasicScrollPaneUI.this.scrollpane.getHorizontalScrollBar();
                    changeListener = BasicScrollPaneUI.this.hsbChangeListener;
                }
                if (changeListener != null) {
                    if (boundedRangeModel != null) {
                        boundedRangeModel.removeChangeListener(changeListener);
                    }
                    if (scrollBar.getModel() != null) {
                        scrollBar.getModel().addChangeListener(changeListener);
                    }
                }
            }
            else if ("componentOrientation" == propertyName && source == BasicScrollPaneUI.this.scrollpane.getHorizontalScrollBar()) {
                JScrollBar horizontalScrollBar = BasicScrollPaneUI.this.scrollpane.getHorizontalScrollBar();
                JViewport viewport = BasicScrollPaneUI.this.scrollpane.getViewport();
                Point viewPosition = viewport.getViewPosition();
                if (BasicScrollPaneUI.this.scrollpane.getComponentOrientation().isLeftToRight()) {
                    viewPosition.x = horizontalScrollBar.getValue();
                }
                else {
                    viewPosition.x = viewport.getViewSize().width - viewport.getExtentSize().width - horizontalScrollBar.getValue();
                }
                viewport.setViewPosition(viewPosition);
            }
        }
    }
}
